// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.


// Code generated from the elasticsearch-specification DO NOT EDIT.
// https://github.com/elastic/elasticsearch-specification/tree/4316fc1aa18bb04678b156f23b22c9d3f996f9c9


// Upgrades a given job snapshot to the current major version.
package upgradejobsnapshot

import (
	gobytes "bytes"
	"context"
	"errors"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"net/url"
	"strconv"
	"strings"

	"github.com/elastic/elastic-transport-go/v8/elastictransport"
)

const (
	jobidMask = iota + 1

	snapshotidMask
)

// ErrBuildPath is returned in case of missing parameters within the build of the request.
var ErrBuildPath = errors.New("cannot build path, check for missing path parameters")

type UpgradeJobSnapshot struct {
	transport elastictransport.Interface

	headers http.Header
	values  url.Values
	path    url.URL

	buf *gobytes.Buffer

	paramSet int

	jobid      string
	snapshotid string
}

// NewUpgradeJobSnapshot type alias for index.
type NewUpgradeJobSnapshot func(jobid, snapshotid string) *UpgradeJobSnapshot

// NewUpgradeJobSnapshotFunc returns a new instance of UpgradeJobSnapshot with the provided transport.
// Used in the index of the library this allows to retrieve every apis in once place.
func NewUpgradeJobSnapshotFunc(tp elastictransport.Interface) NewUpgradeJobSnapshot {
	return func(jobid, snapshotid string) *UpgradeJobSnapshot {
		n := New(tp)

		n.JobId(jobid)

		n.SnapshotId(snapshotid)

		return n
	}
}

// Upgrades a given job snapshot to the current major version.
//
// https://www.elastic.co/guide/en/elasticsearch/reference/current/ml-upgrade-job-model-snapshot.html
func New(tp elastictransport.Interface) *UpgradeJobSnapshot {
	r := &UpgradeJobSnapshot{
		transport: tp,
		values:    make(url.Values),
		headers:   make(http.Header),
		buf:       gobytes.NewBuffer(nil),
	}

	return r
}

// HttpRequest returns the http.Request object built from the
// given parameters.
func (r *UpgradeJobSnapshot) HttpRequest(ctx context.Context) (*http.Request, error) {
	var path strings.Builder
	var method string
	var req *http.Request

	var err error

	r.path.Scheme = "http"

	switch {
	case r.paramSet == jobidMask|snapshotidMask:
		path.WriteString("/")
		path.WriteString("_ml")
		path.WriteString("/")
		path.WriteString("anomaly_detectors")
		path.WriteString("/")
		path.WriteString(url.PathEscape(r.jobid))
		path.WriteString("/")
		path.WriteString("model_snapshots")
		path.WriteString("/")
		path.WriteString(url.PathEscape(r.snapshotid))
		path.WriteString("/")
		path.WriteString("_upgrade")

		method = http.MethodPost
	}

	r.path.Path = path.String()
	r.path.RawQuery = r.values.Encode()

	if r.path.Path == "" {
		return nil, ErrBuildPath
	}

	if ctx != nil {
		req, err = http.NewRequestWithContext(ctx, method, r.path.String(), r.buf)
	} else {
		req, err = http.NewRequest(method, r.path.String(), r.buf)
	}

	req.Header.Set("accept", "application/vnd.elasticsearch+json;compatible-with=8")

	if err != nil {
		return req, fmt.Errorf("could not build http.Request: %w", err)
	}

	return req, nil
}

// Do runs the http.Request through the provided transport.
func (r UpgradeJobSnapshot) Do(ctx context.Context) (*http.Response, error) {
	req, err := r.HttpRequest(ctx)
	if err != nil {
		return nil, err
	}

	res, err := r.transport.Perform(req)
	if err != nil {
		return nil, fmt.Errorf("an error happened during the UpgradeJobSnapshot query execution: %w", err)
	}

	return res, nil
}

// IsSuccess allows to run a query with a context and retrieve the result as a boolean.
// This only exists for endpoints without a request payload and allows for quick control flow.
func (r UpgradeJobSnapshot) IsSuccess(ctx context.Context) (bool, error) {
	res, err := r.Do(ctx)

	if err != nil {
		return false, err
	}
	io.Copy(ioutil.Discard, res.Body)
	err = res.Body.Close()
	if err != nil {
		return false, err
	}

	if res.StatusCode >= 200 && res.StatusCode < 300 {
		return true, nil
	}

	return false, nil
}

// Header set a key, value pair in the UpgradeJobSnapshot headers map.
func (r *UpgradeJobSnapshot) Header(key, value string) *UpgradeJobSnapshot {
	r.headers.Set(key, value)

	return r
}

// JobId Identifier for the anomaly detection job.
// API Name: jobid
func (r *UpgradeJobSnapshot) JobId(v string) *UpgradeJobSnapshot {
	r.paramSet |= jobidMask
	r.jobid = v

	return r
}

// SnapshotId A numerical character string that uniquely identifies the model snapshot.
// API Name: snapshotid
func (r *UpgradeJobSnapshot) SnapshotId(v string) *UpgradeJobSnapshot {
	r.paramSet |= snapshotidMask
	r.snapshotid = v

	return r
}

// WaitForCompletion When true, the API won’t respond until the upgrade is complete.
// Otherwise, it responds as soon as the upgrade task is assigned to a node.
// API name: wait_for_completion
func (r *UpgradeJobSnapshot) WaitForCompletion(b bool) *UpgradeJobSnapshot {
	r.values.Set("wait_for_completion", strconv.FormatBool(b))

	return r
}

// Timeout Controls the time to wait for the request to complete.
// API name: timeout
func (r *UpgradeJobSnapshot) Timeout(value string) *UpgradeJobSnapshot {
	r.values.Set("timeout", value)

	return r
}
